from __future__ import annotations

import bs4
import defopt
import httpx

from .intersphinx import fetch_inv

__all__ = ["lookup_docs", "list_entities", "cli"]

bs = bs4.BeautifulSoup()


def lookup_docs(url_base, inv_objects, debug: bool = False):
    docs = []
    with httpx.Client() as client:
        for obj in inv_objects:
            url = url_base + obj.uri_expanded
            url_head, _, doc_id = url.rpartition("#")
            r = client.get(url)
            # TODO: parse soup
            if debug:
                breakpoint()
            else:
                print(obj)
    return docs


def fetch_entities(
    package_name: str = "pandas",
    *,
    version: str = "",
    debug: bool = False,
    fetch: bool = True,
) -> None:
    print(f"qp ⠶ {package_name=} & {version=}")
    url_base, inv = fetch_inv(package=package_name, version=version)
    py_objects = [o for o in inv.objects if o.domain == "py"]
    print(f"Fetched {len(py_objects)} :py: domain objects from {url_base}")
    if fetch:
        lookup_docs(url_base=url_base, inv_objects=py_objects, debug=debug)
    else:
        if debug:
            breakpoint()
        else:
            for obj in py_objects:
                print(f"{obj.name} {url_base}{obj.uri_expanded}")


def cli():
    defopt.run(fetch_entities)



from __future__ import annotations

from sphobjinv import Inventory

__all__ = ["pkg_urls_and_versions", "fetch_inv"]


# Could set versions with {major}.{minor} if more generality desired
pkg_urls_and_versions = {
    "numpy": ("https://numpy.org/doc/{version}/", "1.23"),
    "pandas": ("https://pandas.pydata.org/pandas-docs/version/{version}/", "1.5"),
    "torch": ("https://pytorch.org/docs/{version}/", "1.12"),
}


def fetch_inv(package: str, version: str) -> tuple[str, dict[str, dict[str, str]]]:
    """
    For reference, see the explanation of the Sphinx objects.inv format given in
    https://buildmedia.readthedocs.org/media/pdf/sphobjinv/v.doc/sphobjinv.pdf
    """
    if package not in pkg_urls_and_versions:
        raise ValueError("No known URL for this package")
    else:
        url_base_template, default_v = pkg_urls_and_versions[package]
        url_base = url_base_template.format(version=version if version else default_v)
        url = url_base + "objects.inv"
    inv = Inventory(url=url)
    return url_base, inv



r"""
:mod:`qp` is...

    >>> import qp
"""

# Sphinx automodule relies on modules being exposed through `__all__` here

__all__ = []  # type: ignore

__author__ = "Louis Maddox"
__license__ = "MIT"
__description__ = "Pandas API reference"
__url__ = "https://github.com/qu-arx/qp"
__uri__ = __url__
__email__ = "louismmx@gmail.com"
